@@ -1,8 +1,29 @@
-el.test.wt <- function(x, wt, mu) {
+eltestwtinC<-function(x,wt,mu){
+	pi=x;
+	Lx=length(x);
+	lam0=0;
+ 	if (sum(is.na(x))>0)stop('NaNs');	
+ 	re=.C('eltestwt',
+		x=as.numeric(x),
+		wt=as.numeric(wt),
+		mu1=mu,
+		Lx1=Lx,
+		pi=as.numeric(pi),
+		lamre=lam0
+	     )
+	return(list(x=re$x, wt=re$wt, prob=re$pi, lam=re$lamre));
+ }
+
+
+el.test.wt <- function(x, wt, mu,usingC=TRUE) {
 #x <- as.matrix(x)
 #if( ncol(x) != 1 ) stop("x must be a vector") 
+
 if( length(mu) != 1 ) stop("mu must be a scalar")
 
+if(usingC){
+return(eltestwtinC(x,wt,mu));
+}else{
 xmu <- x-mu
 allw <- sum(wt)
 BU <- 0.02*allw/max(abs(xmu))
@@ -24,7 +45,7 @@ else {
  lam0 <- uniroot(lamfun,lower=lo,upper=up,tol=1e-9,xmu=xmu,wt=wt,allw=allw)$root
 }
 pi <- wt/(allw + lam0*xmu)
-list(x=x, wt=wt, prob=pi, lam=lam0)
+return(list(x=x, wt=wt, prob=pi, lam=lam0));
+}
 }
-
 ### add output of lam0, 5/2007 
